<?php

/**
 * Registers all actions and filters for the plugin.
 *
 * Maintains a list of all hooks that are registered throughout
 * the plugin, and registers them with the WordPress API. Calls the
 * run function to execute the list of actions and filters.
 *
 * @package    Aqi_Widget
 * @subpackage Aqi_Widget/includes
 * @author     Mid-Ohio Regional Planning Commission <gohio@morpc.org>
 */
class Aqi_Widget_Loader {

	/**
	 * The array of actions registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $actions    The actions registered with WordPress to fire when the plugin loads.
	 */
	protected $actions;

	/**
	 * The array of filters registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $filters    The filters registered with WordPress to fire when the plugin loads.
	 */
	protected $filters;

	/**
	 * The array of shortcodes registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $shortcodes    The shortcodes registered with WordPress to fire when the plugin loads.
	 */
	protected $shortcodes;

	/**
	 * Initializes the collections used to maintain the shortcodes, actions, and filters.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		$this->actions = array();
		$this->filters = array();
		$this->shortcodes = array();
	}

	/**
	 * Adds a new action to the collection to be registered.
	 *
	 * @since    1.0.0
	 * @param    string    $hook             The name of the WordPress action that is being registered.
	 * @param    object    $component        A reference to the instance of the object on which the action is defined.
	 * @param    string    $callback         The name of the function definition on the $component.
	 * @param    int       $priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int       $accepted_args    Optional. The number of arguments that should be passed to the $callback. Default is 1.
	 */
	public function add_action( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->actions = $this->add( $this->actions, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * Adds a new filter to the collection to be registered.
	 *
	 * @since    1.0.0
	 * @param    string    $hook             The name of the WordPress filter that is being registered.
	 * @param    object    $component        A reference to the instance of the object on which the filter is defined.
	 * @param    string    $callback         The name of the function definition on the $component.
	 * @param    int       $priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int       $accepted_args    Optional. The number of arguments that should be passed to the $callback. Default is 1
	 */
	public function add_filter( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->filters = $this->add( $this->filters, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
     * Adds a new shortcode to the collection to be registered.
     *
     * @since     1.0.0
     * @param     string   $tag           The name of the new shortcode.
     * @param     object   $component     A reference to the instance of the object on which the shortcode is defined.
     * @param     string   $callback      The name of the function that defines the shortcode.
     */
    public function add_shortcode( $tag, $component, $callback, $priority = 10, $accepted_args = 1 ) {
        $this->shortcodes = $this->add( $this->shortcodes, $tag, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * A utility function which registers the actions and hooks into a single
	 * collection.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    array     $hooks            The collection of hooks that is being registered (that is, actions or filters).
	 * @param    string    $hook             The name of the WordPress filter that is being registered.
	 * @param    object    $component        A reference to the instance of the object on which the filter is defined.
	 * @param    string    $callback         The name of the function definition on the $component.
	 * @param    int       $priority         The priority at which the function should be fired.
	 * @param    int       $accepted_args    The number of arguments that should be passed to the $callback.
	 * @return   array                       The collection of actions and filters registered with WordPress.
	 */
	private function add( $hooks, $hook, $component, $callback, $priority, $accepted_args ) {
		$hooks[] = array(
			'hook'          => $hook,
			'component'     => $component,
			'callback'      => $callback,
			'priority'      => $priority,
			'accepted_args' => $accepted_args
		);

		return $hooks;
	}

	/**
	 * Registers the the plugins shortcodes, filters, and actions.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		foreach ( $this->filters as $hook ) {
			add_filter( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}

		foreach ( $this->actions as $hook ) {
			add_action( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}

		foreach ( $this->shortcodes as $hook ) {
            add_shortcode( $hook['hook'], array( $hook['component'], $hook['callback'] ) );
        }
	}

}
